/*
 * ouvej, determines what, when and where the next lesson will be
 * Copyright (C) 2003 Olivier Mehani <shtrom@ssji.net, olmehani@etu.utc.fr>
 * 
 * $Id: week-weak.c,v 1.7 2004/03/07 19:55:12 shtrom Exp $
 * weak a/b week sequence handling, based on a hand filled specified file
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
/* in fact with the following, everything works randomly */
#if  0
#define _XOPEN_SOURCE /* see strptime(3) */
#define __USE_XOPEN /* see /usr/include/time.h... */
#endif
#include <time.h>
#include "ouvej.h"

#warning check the proper WEEKSEQFILE has been specified

time_t strip_hours(time_t tt);
week_sequence_t parse_line(char *line);

/*
 * little description of the week file
 * this file contains every (seperated) day along with the week they pertain to
 * as it is humain maintained, it is text oriented, the entries are as follow
 * YYYY-MM-DD<TAB>WS
 * the parsing functions are _absolutely_ NOT fault tolerant:
 * 7 single 12 characters (not including LF) per line
 */
week_sequence_t get_this_week_sequence()
{
	FILE *fp;
	char line[14]; /* 12 + newline + line terminator */
	time_t now, current;
	double diff;
	struct tm t;
	int jmp;
	
	DPRINT("guessing...\n");

	if(!(fp=fopen(WEEKSEQFILE,"r"))) {
		return W_ERROR;
	}
	
	line[13]=0;
	if(fread(line,13,1,fp)<=0) {
		fclose(fp);
		return W_ERROR;
	}

	DPRINT("line: %s",line);
	
	strptime(line,"%Y-%m-%d", &t);
	current=mktime(&t);
	
	DPRINT("in file (without hours): %s",ctime(&current));
	
	now=time(NULL);
	now=strip_hours(now);
	DPRINT("now (without hours): %s",ctime(&now));

	diff=difftime(now,current);
	jmp=(int)(diff/86400); /* 86400 sec per day */
	DPRINT("diff in days: %d\n", jmp);
	
	if(!jmp) {
		DPRINT("today !\n");
	} else if(jmp<0) {
		fclose(fp);
		return W_ERROR;
	} else {
		DPRINT("seeking...\n");
		/* we already read a line, and we want to reach the point
		 * _before_ the line we're interested in, so we have to seek 2
		 * lines less than the difference we found */
		/*jmp-=2;*/
		if(fseek(fp,jmp*13,SEEK_SET)) {
			DPRINT("error seeking\n");
			fclose(fp);
			return W_ERROR;
		}
		if(fread(line,13,1,fp)<=0) {
			DPRINT("error reading");
			fclose(fp);
			return W_ERROR;
		}
	}
	
	DPRINT("%s",line);

	fclose(fp);

	return parse_line(line);
}

time_t strip_hours(time_t tt)
{
	struct tm *t;
	t=gmtime(&tt);
	/* 
	 * don't do this at home kids... 
	 * in fact, don't do anything you've seen in this file
	 * _they_ forced me to write it
	 *
	 * i didn't want
	 *
	 * _they_'re everywhere
	 *
	 * we're _all_ gonna DIE
	 */
	t->tm_sec = t->tm_min = t->tm_hour = 12;
	return mktime(t);
}

week_sequence_t parse_line(char *line) 
{
	/* remember the file is very fixed :-) */
	DPRINT("week char: %c\n",line[11]);
	switch(line[11]) {
	case 'A':
		return W_A;
	case 'B':
		return W_B;
	default:	
		return W_N;
	}
}
